#####################################################################
# 
# TQPollingThread.py - Zope Transactional Queue Manager
#
# $Id: TQPollingThread.py,v 1.1 2003/11/19 18:37:42 kevin Exp $
#
# Copyright (C) 2003 Kevin Gill
#
# Derived from the Xron version 0.0.10
# Copyright (C) 2000 Loren Stafford
#  
# Xron is Derived from the ZScheduler product version 0.0.7 in
# accord with the terms of the ZScheduler's license.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the 
#     Free Software Foundation, Inc., 
#     59 Temple Place - Suite 330, 
#     Boston, MA  02111-1307, USA.
# 
#####################################################################
""" Zope Transactional Queue - Polling Thead

The transaction queue system includes a background thread
which can automatically dequeue items and feed them into
a configured process. 

To find all the transaction queues in the system requires
a single central list. This Polling Thread is that list. It
should also provide to ZMI a couple of screens for 
tracking the behaviour of the background thread.
"""

import TQInterface

from ZODB.PersistentList import PersistentList
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from Globals import InitializeClass
import Acquisition
from DateTime import DateTime

import zLOG

# This is the name, and instance for the object in the root directory
GlobalID='TQPollingThread'


class TQPollingThread(Item, PropertyManager, Acquisition.Implicit, PersistentList, Role.RoleManager):
    """Cheap and dirty implementation. I use a persistent mapping to store
    the queue, and the keys are sequential to maintain the queue ordering."""

    def registerObject(self, obj):
        """Adds a new object to the system"""
        try:
            path = obj.path
        except:
            path = obj
        if self.count(path) == 0:
            self.append(path)
    def unregisterObject(self, obj):
        """Drops a queue from the system"""
        try:
            path = obj.path
        except:
            path = obj
        while self.count(path) > 0:
            self.remove(path)
    def reregisterObject(self, old_path, obj):
        """A queue has been renamed or moved - only detected on an insert"""
        try:
            new_path = obj.path
        except:
            new_path = obj
        while self.count(old_path) > 0:
            self.remove(old_path)
        if self.count(new_path) == 0:
            self.append(new_path)

    def items(self):
        """Returns the list of queues - for ZMI. """
        rv = []
        for item in self:
            try:
                instance = self.unrestrictedTraverse(item)
                rv.append({'path': item, 'status': instance.getStatusMessage()})
                del instance
            except:
                pass
        return rv

    def __init__(self, id, title):
        """Initialise"""
        PersistentList.__init__(self)
        self.id = id
        self.title = title
        self.minInterval = 100
        self.maxInterval = 10000

    ### Consider everything beyond this private

    __ac_roles__ = ()
    __implements__ = TQInterface.IPollingThread
    meta_type = 'TQPollingThread'
    security = ClassSecurityInfo()

    security.declareProtected('TQPollingThread Modify', 'registerObject')
    security.declareProtected('TQPollingThread Modify', 'unregisterObject')
    security.declareProtected('TQPollingThread Modify', 'reregisterObject')
    security.declarePublic('items')
    security.setPermissionDefault('TQPollingThread Modify', ('Manager', 'Owner'))

    manage_options = ((
       {'label':'Contents', 'action':'manage_main', 'help': ('ZTxnQueue', 'TQPollingThread.stx')},)+ 
       PropertyManager.manage_options+Role.RoleManager.manage_options+ Item.manage_options)
        
    manage_main = PageTemplateFile('www/manage_pollingthreadView.zpt', globals())
    _properties = ({'id': 'title', 'type': 'string', 'mode': 'w'},
                   {'id': 'minInterval', 'type': 'int', 'mode': 'w'},
                   {'id': 'maxInterval', 'type': 'int', 'mode': 'w'},
    )

InitializeClass(TQPollingThread)

